package se.kawaiiriver;

import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.graphics.FontData;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Dialog;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.ProgressBar;
import org.eclipse.swt.widgets.Shell;

public class GenericProgressDialog extends Dialog {
	private Shell shell;
	private Label header, status;
	private ProgressBar progressBar;
	private String headerStr;

	
	public GenericProgressDialog(Shell shell, int style, String headerStr) {
		super(shell, style);
		this.headerStr = headerStr;
	}
	
	public void open() {
		shell = new Shell(getParent(), SWT.DIALOG_TRIM | SWT.APPLICATION_MODAL);
		shell.setText(headerStr);
		GridLayout gridLayout = new GridLayout();
		gridLayout.horizontalSpacing = 6;
		gridLayout.verticalSpacing = 6;
		gridLayout.marginHeight = 12;
		gridLayout.marginWidth = 12;
		shell.setLayout(gridLayout);
		
		header = new Label(shell, SWT.NONE);
		header.setText(headerStr);
		FontData[] fontData = header.getFont().getFontData();
		fontData[0].setStyle(SWT.BOLD);
		fontData[0].setHeight(fontData[0].getHeight() + 4);
		header.setFont(new Font(Display.getDefault(), fontData));
		
		progressBar = new ProgressBar(shell, SWT.HORIZONTAL | SWT.SMOOTH);
		progressBar.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		
		status = new Label(shell, SWT.NONE);
		status.setText("Getting how many titles are on the DVD");
		fontData = status.getFont().getFontData();
		fontData[0].setStyle(SWT.ITALIC);
		status.setFont(new Font(getParent().getDisplay(), fontData));
		
		shell.pack();
		shell.setSize(400, shell.getSize().y);
		
		shell.open();
		while (!shell.isDisposed())
			if (!getParent().getDisplay().readAndDispatch())
				getParent().getDisplay().sleep();
	}
	
	public synchronized void close() {
		Display.getDefault().syncExec(new Runnable() {
			public void run() {
				shell.dispose();
			}
		});
	}
	
	public synchronized void setMaximumPosition(final int max) {
		Display.getDefault().syncExec(new Runnable() {
			public void run() {
				progressBar.setMaximum(max);
			}
		});
	}
	
	public synchronized void setPosition(final int progress) {
		Display.getDefault().syncExec(new Runnable() {
			public void run() {
				progressBar.setSelection(progress);
			}
		});
	}	
	public synchronized void setStatus(final String statusString) {
		Display.getDefault().syncExec(new Runnable() {
			public void run() {
				status.setText(statusString);
				status.pack();
			}
		});
	}	
}
