/* Matroska Subtitle Patch
 * (c) David Grundberg (c04dgg@cs.umu.se)
 * All rights reserved.
 */

package se.kawaiiriver;

import java.io.*;
import java.util.*;
import java.util.regex.*;

public class MPlayerInfo {
	private Process proc;

	private StringBuffer mplayerOutput;

	private boolean commandFound = true;

	public MPlayerInfo(String video) {
		this(video, null);
	}

	public MPlayerInfo(String video, String dvdDrive) {

		String[] command = null;
		// Use md5sum as video output instead of the null one, then we get movie
		// aspect even for dvd:s...
		if (dvdDrive != null)
			command = new String[] { MPlayerInfo.getMPlayerExecutable(),
					"-vo", "md5sum", "-ao", "null", "-frames", "1",
					"-dvd-device", dvdDrive, video.toString(), "-v",
					"-identify" };
		else
			command = new String[] { MPlayerInfo.getMPlayerExecutable(),
					"-vo", "md5sum", "-ao", "null", "-frames", "1",
					video.toString(), "-identify" };

		String commandStr = "";
		for (int i = 0; i < command.length; i++)
			commandStr += command[i] + " ";
		Logger.logMessage(commandStr, Logger.INFO);

		mplayerOutput = new StringBuffer();

		try {
			proc = Runtime.getRuntime().exec(command);
		} catch (IOException io) {
			io.printStackTrace();
		}
		// We'll need to read the error stream too, otherwise mplayer may stall.
		BlackHole.suckUp(proc.getErrorStream());

		BufferedReader input = null;
		try {
			input = new BufferedReader(new InputStreamReader(proc
					.getInputStream()));
			String line;
			while ((line = input.readLine()) != null) {
				mplayerOutput.append(line + "\n");
				Logger.logMessage(line, Logger.MPLAYER);
			}

		} catch (IOException io) {
			io.printStackTrace();
		} finally {
			if (input != null) {
				try {
					input.close();
				} catch (IOException e) {
				}
			}
		}

		try {
			proc.waitFor();
		} catch (InterruptedException e) {
		}

	}

	public String getVideoFormat() {
		Matcher matcher = Pattern.compile("ID_VIDEO_FORMAT=.*").matcher(
				mplayerOutput);
		matcher.find();
		String output = "";

		try {
			output = matcher.group();
		} catch (Exception e) {
			// empty
		}

		return output.substring(output.indexOf('=') + 1);
	}

	public int getLength() {
		Matcher matcher = Pattern.compile("ID_LENGTH=[0-9]*").matcher(
				mplayerOutput);
		matcher.find();
		String output = "";

		try {
			output = matcher.group();
		} catch (Exception e) {
			// empty
		}

		try {
			return Integer.parseInt(output.substring(output.indexOf('=') + 1));
		} catch (Exception e) {
			return 0;
		}
	}

	public int getNumberOfTitles() {
		Matcher matcher = Pattern.compile("ID_DVD_TITLES=[0-9]*").matcher(
				mplayerOutput);
		matcher.find();
		String output = matcher.group();

		return Integer.parseInt(output.substring(14));
	}

	public int getNumberOfChapters() {
		Matcher matcher = Pattern.compile("There are [0-9]* chapters").matcher(
				mplayerOutput);
		matcher.find();
		String output = matcher.group();

		// originally used String.split() not available in GCJ
		return Integer.parseInt(output.substring(10, output
				.indexOf(" chapters")));
	}

	public Map getAudioStreams() {
		Map languages = new LinkedHashMap();

		languages.put("Default", "-1");

		Matcher matcher;

		/*
		 * = Pattern.compile("audio stream: [0-9]* audio format:
		 * ac3.*").matcher(mplayerOutput); while (matcher.find()) { String
		 * stream = matcher.group(); String lang =
		 * (Integer.parseInt(stream.substring(stream.indexOf("stream: ") + 8,
		 * stream.indexOf(" audio format:"))) + 1) + ". " +
		 * stream.substring(stream.indexOf("language: ") + 10, stream.indexOf("
		 * aid:")); languages.put(lang, stream.substring(stream.indexOf(" aid: ") +
		 * 6)); }
		 */

		// Parse embedded audio streams (matroska)
		/*
		 * if (languages.size() == 1) { matcher = Pattern.compile(" -aid
		 * ([0-9]{1,})[ ,-]* -alang ([a-z]{2,})").matcher(mplayerOutput); while
		 * (matcher.find()) { String aid = matcher.group(1); String lang =
		 * matcher.group(2); languages.put(lang, aid); } }
		 */

		// Parse embedded audio streams (tested with matroska, ogg vorbis)
		if (languages.size() == 1) {
			matcher = Pattern.compile("^ID_AUDIO_ID=([0-9]+)$",
					Pattern.MULTILINE).matcher(mplayerOutput);
			while (matcher.find()) {
				String aid = matcher.group(1);

				// Find out audio stream language if available
				// Default string in case there is no corresponding
				// ID_AID_x_LANG (can happen)
				String description = "Unknown";
				Matcher descMatcher = Pattern.compile(
						"^ID_AID_" + aid + "_LANG=(.*)$", Pattern.MULTILINE)
						.matcher(mplayerOutput);
				if (descMatcher.find()) {
					description = descMatcher.group(1);
				}

				// Add format info if DVD (hack)
				descMatcher = Pattern.compile(
						"^audio stream: .* format: (.*) language: .* aid: "
								+ aid + ".$", Pattern.MULTILINE).matcher(
						mplayerOutput);
				if (descMatcher.find()) {
					description = description + " " + descMatcher.group(1);
				}
				description = aid + ": " + description;
				// System.out.println(aid+":---"+lang);

				languages.put(description, aid);
			}

		}

		return languages;
	}

	public Map getSubtitleLanguages() {
		Map languages = new LinkedHashMap();

		languages.put("None", "-1");
		/*
		 * Matcher matcher = Pattern.compile("[0-9]{1,} language:
		 * [a-z]{2}").matcher(mplayerOutput); while (matcher.find()) { String
		 * sub = matcher.group(); String lang =
		 * (Integer.parseInt(sub.substring(0, sub.indexOf(' '))) + 1) + ". " +
		 * sub.substring(sub.indexOf(": ") + 2); languages.put(lang,
		 * sub.substring(0, sub.indexOf(' '))); }
		 */

		/*
		 * // Parse embedded subtitles (matroska) if (languages.size() == 1) {
		 * matcher = Pattern.compile(" -sid ([0-9]{1,})[ ,-]* -slang
		 * ([a-z]{2,})").matcher(mplayerOutput); while (matcher.find()) { String
		 * sid = matcher.group(1); String lang = matcher.group(2);
		 * languages.put(lang, sid); } }
		 */
		Matcher matcher;
		// Parse embedded subtitles (tested with matroska, ogg vorbis)
		if (languages.size() == 1) {
			matcher = Pattern.compile("^ID_SUBTITLE_ID=([0-9]+)$",
					Pattern.MULTILINE).matcher(mplayerOutput);
			while (matcher.find()) {
				String sid = matcher.group(1);
				String description = "Unknown";

				Matcher descMatcher = Pattern.compile(
						"^ID_SID_" + sid + "_LANG=(.*)$", Pattern.MULTILINE)
						.matcher(mplayerOutput);
				if (descMatcher.find()) {
					description = descMatcher.group(1);
				}

				// Add name if found (I've seen Matroska use these) Here's an
				// example:
				// ID_SUBTITLE_ID=0
				// ID_SID_0_NAME=Normal Subtitles
				// ID_SID_0_LANG=eng
				// [mkv] Track ID 4: subtitles (S_TEXT/SSA) "Normal Subtitles",
				// -sid 0, -slang eng
				// ID_SUBTITLE_ID=1
				// ID_SID_1_NAME=Subtitles with Karaoke
				// ID_SID_1_LANG=eng
				// [mkv] Track ID 5: subtitles (S_TEXT/SSA) "Subtitles with
				// Karaoke", -sid 1, -slang eng

				descMatcher = Pattern.compile("^ID_SID_" + sid + "_NAME=(.*)$",
						Pattern.MULTILINE).matcher(mplayerOutput);
				if (descMatcher.find()) {
					description = description + ": " + descMatcher.group(1);
				}

				// hack: Add matroska subtitle format info.
				String t = "^\\[mkv\\] Track ID .*: subtitles \\((.*)\\).* -sid "
						+ sid + ",.*$";
				System.out.println("muh: " + t);
				descMatcher = Pattern.compile(t, Pattern.MULTILINE).matcher(
						mplayerOutput);
				if (descMatcher.find()) {
					description = description + " (" + descMatcher.group(1)
							+ ")";
				}

				description = sid + ": " + description;
				// System.out.println(sid+":---"+descr);

				languages.put(description, sid);
			}

		}

		return languages;
	}

	public double getFrameRate() {
		double frameRate = 0;
		Matcher matcher = Pattern.compile("([0-9.]+) fps").matcher(
				mplayerOutput);

		if (!matcher.find())
			return -1;

		frameRate = Double.parseDouble(matcher.group(1));

		return frameRate;
	}

	public Dimensions getDimensions() {

		Matcher matcher = Pattern.compile("ID_VIDEO_WIDTH=(\\d+)").matcher(
				mplayerOutput);

		if (matcher.find()) {
			int width = Integer.parseInt(matcher.group(1));
			matcher = Pattern.compile("ID_VIDEO_HEIGHT=(\\d+)").matcher(
					mplayerOutput);
			if (matcher.find()) {
				int height = Integer.parseInt(matcher.group(1));
				matcher = Pattern
						.compile(
								"Movie-Aspect is (\\d+(?:\\.\\d+)?):1 - prescaling to correct movie aspect.")
						.matcher(mplayerOutput);
				if (matcher.find()) {
					double scale = Double.parseDouble(matcher.group(1));
					width = (int) Math.round(height * scale);
				}
				return new Dimensions(width, height);
			}

		}

		matcher = Pattern.compile("=> [0-9]*x[0-9]*").matcher(mplayerOutput);

		if (!matcher.find())
			return new Dimensions(-1, -1);

		return new Dimensions(matcher.group().substring(
				matcher.group().indexOf(' ') + 1));
	}
	
	public static String getMPlayerExecutable() {
		return findExecutable("mplayer");
	}

	public static String getMEncoderExecutable() {
		return findExecutable("mencoder");
	}


	private static String findExecutable(String basename) {
		File currentDirectory = new File(Config.getPackageDataDir(), "mplayer");
		String[] files = currentDirectory.list();
		File executable = null;

		String basenameWindows = basename+".exe";
		for (int i = 0; files != null && i < files.length; i++) {
			if (files[i].equals(basename) || files[i].equals(basenameWindows))	{
				executable = new File(currentDirectory,files[i]);
			}
		}
			

		if (executable == null)
			throw new RuntimeException("Executable "+basename+" or "+basenameWindows+" not found in: "+currentDirectory);
		
		return executable.toString();
	}

	public boolean commandFound() {
		return commandFound;
	}

	public boolean videoSupported() {
		if (getFrameRate() == -1 || getDimensions().getWidth() == -1
				|| getDimensions().getHeight() == -1)
			return false;

		return true;
	}
}
