/* Matroska Subtitle Patch
 * (c) David Grundberg (c04dgg@cs.umu.se)
 * All rights reserved.
 */

package se.kawaiiriver;

import org.eclipse.swt.*;
import org.eclipse.swt.widgets.*;
import org.eclipse.swt.events.*;
import org.eclipse.swt.layout.*;
import org.eclipse.swt.custom.*;
import org.eclipse.swt.graphics.*;

import java.io.*;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Map;

public class SingleVideo extends Composite implements SelectionListener, TabItemControl{
	private CTabItem tabItem;
	private Text inputVideoInput, outputVideoInput;
	private Button inputVideoSelect, outputVideoSelect, fetchSubtitles;
	private String syncInputVideo, syncOutputVideo;
	private Combo audioStreamCombo, subtitlesCombo;
	protected int syncAudioStream;
	protected int syncSubtitles;
	private Map audioStreams;
	private Map subtitles;
	private Button previewButton;
		
	public SingleVideo(Composite parent, int style, CTabItem tabItem) {
		super(parent, style);
		this.tabItem = tabItem;
		
		InputStream is = getClass().getResourceAsStream("icons/singlevideo-16.png");
		tabItem.setImage(new Image(getDisplay(), is));
		tabItem.setText("New Single Video");
		
		GridLayout gridLayout = new GridLayout();
		gridLayout.horizontalSpacing = 6;
		gridLayout.verticalSpacing = 6;
		gridLayout.marginHeight = 12;
		gridLayout.marginWidth = 12;
		// gridLayout.numColumns = 4;
		gridLayout.numColumns = 3;
		setLayout(gridLayout);
		
		Label singleVideoLabel = new Label(this, SWT.NONE);
		singleVideoLabel.setText("Single Video");
		FontData[] fontData = singleVideoLabel.getFont().getFontData();
		fontData[0].setStyle(SWT.BOLD);
		singleVideoLabel.setFont(new Font(getParent().getDisplay(), fontData));
		GridData gridData = new GridData();
		// gridData.horizontalSpan = 4;
		gridData.horizontalSpan = 3;
		singleVideoLabel.setLayoutData(gridData);
		
		/* Label tab = new Label(this, SWT.NONE);
		tab.setText("\t"); */
		
		Label inputVideo = new Label(this, SWT.NONE);
		inputVideo.setText("Input:");
		
		inputVideoInput = new Text(this, SWT.BORDER);
		inputVideoInput.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		
		inputVideoSelect = new Button(this, SWT.PUSH);
		inputVideoSelect.setText("Select");
		gridData = new GridData();
		gridData.widthHint = 75;
		inputVideoSelect.setLayoutData(gridData);
		inputVideoSelect.addSelectionListener(this);
		
		/* tab = new Label(this, SWT.NONE);
		tab.setText("\t"); */
		
		Label outputVideo = new Label(this, SWT.NONE);
		outputVideo.setText("Output:");
		
		outputVideoInput = new Text(this, SWT.BORDER);
		outputVideoInput.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		
		outputVideoSelect = new Button(this, SWT.PUSH);
		outputVideoSelect.setText("Select");
		gridData = new GridData();
		gridData.widthHint = 75;
		outputVideoSelect.setLayoutData(gridData);
		outputVideoSelect.addSelectionListener(this);

		fetchSubtitles = new Button(this, SWT.PUSH);
		fetchSubtitles.setText("Fetch subtitles");
		fetchSubtitles.addSelectionListener(this);
		
		
		Composite groupsComp = new Composite(this, SWT.NONE);
		gridLayout = new GridLayout();
		gridLayout.horizontalSpacing = 6;
		gridLayout.verticalSpacing = 6;
		gridLayout.marginHeight = 0;
		gridLayout.marginWidth = 0;
		gridLayout.numColumns = 2;
		gridLayout.makeColumnsEqualWidth = true;
		groupsComp.setLayout(gridLayout);
		groupsComp.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));

		
		
		Group languages = new Group(groupsComp, SWT.NONE);
		languages.setText("Languages");
		gridLayout = new GridLayout();
		gridLayout.horizontalSpacing = 6;
		gridLayout.verticalSpacing = 6;
		gridLayout.numColumns = 2;
		languages.setLayout(gridLayout);
		languages.setLayoutData(new GridData(GridData.FILL_BOTH));
		
		Label audioStreamLabel = new Label(languages, SWT.NONE);
		audioStreamLabel.setText("Audio Stream:");
		
		audioStreamCombo = new Combo(languages, SWT.DROP_DOWN | SWT.READ_ONLY);
		audioStreamCombo.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));

		
		Label subtitlesLabel = new Label(languages, SWT.NONE);
		subtitlesLabel.setText("Subtitles:");
		
		subtitlesCombo = new Combo(languages, SWT.DROP_DOWN | SWT.READ_ONLY);
		subtitlesCombo.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		
		previewButton = new Button(languages, SWT.PUSH);
		previewButton.setText("Preview");
		gridData = new GridData(GridData.FILL_HORIZONTAL);
		gridData.horizontalSpan = 2;
		previewButton.setLayoutData(gridData);
		previewButton.addSelectionListener(this);
		
		
		// Initiate subtitle and audio stream controls.
		resetSubtitles();
		
				//fetchSubtitles.setLayoutData(gridData);
		

	}
	
	public void widgetDefaultSelected(SelectionEvent e) {
		// empty
	}
	
	public void widgetSelected(SelectionEvent e) {
		if (e.getSource() == inputVideoSelect) {
			FileDialog fileDialog = new FileDialog(getShell(), SWT.OPEN);
			fileDialog.setText("Input Video");
			fileDialog.setFilterExtensions(new String[]{"*.avi;*.vob;*.mkv;*.mpg;*.mpeg;*.ogm;*.mov;*.rm;*.ram;*.wmv;*.asf", "*.avi", "*.vob", "*.mkv", "*.mpg;*.mpeg;*.mp4", "*.ogm", "*.mov", "*.rm;*.ram", "*.wmv;*.asf", "*"});
			fileDialog.setFilterNames(new String[]{"All Video Files", "AVI Video (*.avi)", "DVD Video Object (*.vob)", "Matroska Video (*.mkv)", "MPEG Video (*.mpg, *.mpeg, *.mp4)", "Ogg Video (*.ogm)", "Quicktime Movie (*.mov)", "Real Video (*.rm, *.ram)", "Windows Media Video (*.wmv, *.asf)", "All Files"});
			String file = fileDialog.open();
			if (file != null) {
				inputVideoInput.setText(file);
				outputVideoInput.setText(file.substring(0, file.lastIndexOf('.')) + "." + ConverterOptions.getCurrentProfile().getProfileName() + "." + ConverterOptions.getCurrentProfile().getWrapperFormat());
				tabItem.setText(new File(file).getName());
				resetSubtitles();
			}
		}
		
		if (e.getSource() == outputVideoSelect) {
			FileDialog fileDialog = new FileDialog(getShell(), SWT.SAVE);
			fileDialog.setText("Output Video");
			if (ConverterOptions.getCurrentProfile().getWrapperFormat().equals("mp4")) {
				fileDialog.setFilterExtensions(new String[]{"*.mp4"});
				fileDialog.setFilterNames(new String[]{"MP4 Video (*.mp4)"});
			} else {
				fileDialog.setFilterExtensions(new String[]{"*.avi"});
				fileDialog.setFilterNames(new String[]{"AVI Video (*.avi)"});
			}
			String file = fileDialog.open();
			if (file != null)
				outputVideoInput.setText(file);
		}
		
		if (e.getSource() == fetchSubtitles) {
			// Show progressbar and retrieve subtitle/audio streams info
			
			
			resetSubtitles(); // Just in case we don't get any information this time
			
			
			if (getInputVideo().trim().length() == 0) {
				MessageBox b = new MessageBox(this.getShell(), SWT.OK | SWT.ICON_INFORMATION);
				b.setText("No file selected");
				b.setMessage("Please select an input video file.");
				b.open();
				return;
			}
			
			GenericProgressDialog pd = new GenericProgressDialog(getShell(),SWT.NONE,"Reading file");
			SingleVideoInfoReader reader = new SingleVideoInfoReader(pd,getInputVideo());
			reader.start();
			pd.open();
			
			MPlayerInfo info = reader.getInfo();
			if (info == null) {
//				 User abort. TODO: Should stop mencoder process?
				return; 
			}
			if (!info.videoSupported()) {
				// Mencoder couldn't understand the video file, stop.
				MessageBox b = new MessageBox(this.getShell(), SWT.OK | SWT.ICON_ERROR);
				b.setText("File couldn't be read");
				b.setMessage("An error occurred while reading the input file.");
				b.open();
				return;
			}

			audioStreams = info.getAudioStreams();
			subtitles = info.getSubtitleLanguages();
			audioStreamCombo.removeAll();
			for (int i = 0; i < audioStreams.keySet().size(); i++)
				audioStreamCombo.add((String) audioStreams.keySet().toArray()[i]);
			
			// select first language if available, default otherwise
			audioStreamCombo.select(audioStreams.size() > 1 ? 1 : 0);
			
			subtitlesCombo.removeAll();
			for (int i = 0; i < subtitles.keySet().size(); i++)
				subtitlesCombo.add((String) subtitles.keySet().toArray()[i]);
			
			subtitlesCombo.select(subtitles.size() > 1 ? 1 : 0);	
			subtitlesCombo.setEnabled(true);
			audioStreamCombo.setEnabled(true);
			
		}
		
		if (e.getSource() == previewButton) {
			try {
				if (getInputVideo().length() == 0) {
					MessageBox b = new MessageBox(this.getShell(), SWT.OK | SWT.ICON_INFORMATION);
					b.setText("No file selected");
					b.setMessage("Please select an input video file.");
					b.open();
					return;
				}
				java.util.List commandList = new ArrayList();
				commandList.add(MPlayerInfo.getMPlayerExecutable());
				commandList.add(getInputVideo());
				
				if (getAudioStream() > -1) {
					commandList.add("-aid");
					commandList.add("" + getAudioStream());
				}

				if (getSubtitle() > -1) {
					commandList.add("-sid");
					commandList.add("" + getSubtitle());
				}
		
				String commandStr = "";
				String[] command = new String[commandList.size()];
				for (int i = 0; i < command.length; i++) {
					command[i] = (String) commandList.get(i);
					commandStr += command[i] + " ";
				}
				Logger.logMessage(commandStr, Logger.INFO);
				
				Process proc = Runtime.getRuntime().exec(command);
				
				BlackHole.suckUp(proc.getErrorStream());
				BlackHole.suckUp(proc.getInputStream());
				
				
				/*proc.getOutputStream().close();
				proc.getErrorStream().close();
				proc.getInputStream().close();*/
			} catch (IOException io) {
				io.printStackTrace();
			}
		}
	

	}
	
	/**
	 * Reset subtitle and audio GUI controls to their default, disabled state.
	 *
	 */
	private void resetSubtitles() {
		subtitlesCombo.removeAll();
		// "None" == magic string, returned in map by: 
		//     MPlayerInfo.getSubtitleLanguages() too
		subtitlesCombo.add("None");
		subtitlesCombo.setEnabled(false); // until user pushes fetchSubtitles
		subtitlesCombo.select(0);
		subtitles = new HashMap(1);
		subtitles.put("None","-1");
		
		audioStreamCombo.removeAll();		
		audioStreamCombo.add("Default"); // magic string
		audioStreamCombo.setEnabled(false); // until user pushes fetchSubtitles
		audioStreamCombo.select(0);
		audioStreams = new HashMap(1);
		audioStreams.put("Default","-1");
	}

	public void setTabItem(CTabItem tabItem) {
		this.tabItem = tabItem;
	}
	
	public void setInputVideo(String inputVideo) {
		tabItem.setText(new File(inputVideo).getName());
		
		inputVideoInput.setText(inputVideo);
		outputVideoInput.setText(inputVideo.substring(0, inputVideo.lastIndexOf('.')) + "." + ConverterOptions.getCurrentProfile().getProfileName() + ".avi");
	}
	
	private String getInputVideo() {		
		return inputVideoInput.getText();
	}
	
	private String getOutputVideo() {
		return outputVideoInput.getText();
	}
	
	/*public synchronized void setOutputVideo(String outputVideo) {
		syncOutputVideo = outputVideo;

		Display.getDefault().syncExec(new Runnable() {
			public void run() {
				outputVideoInput.setText(syncOutputVideo);
		});
			}
	}*/

	private int getAudioStream() {
		return Integer.parseInt((String)audioStreams.get(audioStreamCombo.getText()));
	}
	
	private int getSubtitle() {
		return Integer.parseInt((String) subtitles.get(subtitlesCombo.getText()));
	}
	
	public SingleVideoInfo getSingleVideoInfo() {
		final SingleVideoInfo[] info = new SingleVideoInfo[1];
		info[0] = null;

		Display.getDefault().syncExec(new Runnable() {
			public void run() {
				info[0]= new SingleVideoAdapter(getInputVideo(),getOutputVideo(),getAudioStream(),getSubtitle());
			}
		});
		
		return info[0];
	}
	
	}

